@@ -9,6 +9,7 @@ test_that("pwelch", {
     window <- hanning(window)
     window_len <- length(window)
     step <- max(floor(window_len - noverlap + 0.99), 1)
+    print(c(1, x_len - window_len + 1, by = step))
     offset <- seq(1, x_len - window_len + 1, by = step)
     N <- length(offset)
     re <- sapply(seq_len(N), function(i) {
@@ -16,17 +17,17 @@ test_that("pwelch", {
       a <- fftwtools::fftw_r2c(postpad(a$Y * window, nfft))
       Mod(a)^2
     })
-    NN <- floor((nfft + 1)/2)
+    NN <- ceiling((nfft + 1)/2)
     freq <- seq(0, fs/2, length.out = NN)
-    spec <- rowMeans(re[seq_len(NN), , drop = F])/(window_len/2)^2
+    spec <- rowMeans(re[seq_len(NN), , drop = F])/ window_len #(window_len/2)^2
     res <- list(freq = freq, spec = spec, method = "Welch")
     return(invisible(res))
   }
 
 
   x <- rnorm(10000)
-  a <- pwelch(x, 100, plot = FALSE)
-  b <- rave_pwelch(x, 100, log = '')
+  a <- pwelch(x, fs = 100, window = 64, noverlap = 6, nfft = 256, plot = FALSE, window_family = hanning)
+  b <- rave_pwelch(x, fs = 100, window = 64, noverlap = 6, nfft = 256, log = '')
 
   expect_equal(a$freq, b$freq)
   expect_equal(a$spec, b$spec)
